@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: jan  8 2019 (11:54) 
 ## Version: 
-## Last-Updated: mar  3 2020 (10:53) 
+## Last-Updated: apr 26 2020 (15:50) 
 ##           By: Brice Ozenne
-##     Update #: 102
+##     Update #: 142
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -20,6 +20,7 @@ if(FALSE){
     library(BuyseTest)
     library(data.table)
 }
+
 ## options("stringsAsFactors" = TRUE)
 context("Check correct computation of the variance \n")
 var2 <- function(x){var(x)*(length(x)-1)/length(x)}
@@ -50,8 +51,8 @@ test_that("iid: binary and no strata (balanced groups)", {
     expect_equal(e.BT@covariance, e2.BT@covariance)
     expect_equal(as.double(e.BT@covariance), c(1/16,1/16,-1/16, 1/4, 4) )
 
-    expect_equal(iid(e.BT)[,"favorable"], c(-1/8,1/8,1/8,-1/8))
-    expect_equal(iid(e.BT)[,"unfavorable"], c(1/8,-1/8,-1/8,1/8))
+    expect_equal(getIid(e.BT)[,"favorable"], c(-1/8,1/8,1/8,-1/8))
+    expect_equal(getIid(e.BT)[,"unfavorable"], c(1/8,-1/8,-1/8,1/8))
 
     ## second order
     BuyseTest.options(order.Hprojection = 2)
@@ -65,10 +66,9 @@ test_that("iid: binary and no strata (balanced groups)", {
                        data = d, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
 
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair,1:4) ## correct ordering of the pairs
     expect_equal(e.BT@covariance, e1.BT@covariance) ## assumes vs. does not assumes binary score when computing second order terms
     expect_equal(e.BT@covariance, e2.BT@covariance) 
-    expect_equal(as.double(e.BT@covariance), c(5/64,5/64,-3/64, 1/4, 4) )
+    expect_equal(as.double(e.BT@covariance), c(5/64, 5/64, -3/64, 1/4, 4) )
 })
 
 ## unequal number in each group
@@ -85,10 +85,10 @@ test_that("iid: binary and no strata (unbalanced groups)", {
                        method.inference = "u-statistic-bebu"))
     
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(as.double(e.BT@covariance), c(0,2/27,0,2/27,0) )
+    expect_equal(as.double(e.BT@covariance), c(0, 2/27, 0, 2/27, 0) )
 
-    expect_equal(iid(e.BT)[,"favorable"], c(0,0,0,0))
-    expect_equal(iid(e.BT)[,"unfavorable"], c(0,-1/9,-1/9,2/9))
+    expect_equal(getIid(e.BT)[,"favorable"], c(0,0,0,0))
+    expect_equal(getIid(e.BT)[,"unfavorable"], c(0,-1/9,-1/9,2/9))
 
     ## second order
     BuyseTest.options(order.Hprojection = 2)
@@ -102,10 +102,9 @@ test_that("iid: binary and no strata (unbalanced groups)", {
                        data = d.bis, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu"))
     
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair,1:3) ## correct ordering of the pairs
     expect_equal(e.BT@covariance, e1.BT@covariance) ## assumes vs. does not assumes binary score when computing second order terms
     expect_equal(e.BT@covariance, e2.BT@covariance) 
-    expect_equal(as.double(e.BT@covariance), c(0,2/27,0,2/27,0) )
+    expect_equal(as.double(e.BT@covariance), c(0, 2/27, 0, 2/27, 0) )
 })
 
 
@@ -119,16 +118,16 @@ test_that("iid: binary with strata (balanced groups)", {
     BuyseTest.options(order.Hprojection = 1)
     e.BT <- BuyseTest(group ~ bin(toxicity) + strata,
                       data = d2, 
-                      method.inference = "u-statistic")
+                      method.inference = "u-statistic", keep.pairScore = TRUE)
     e2.BT <- BuyseTest(group ~ bin(toxicity) + strata,
                        data = d2, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
 
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(as.double(e.BT@covariance), c(1/16,1/16,-1/16,1/4,4)/3 )
+    expect_equal(as.double(e.BT@covariance), c(1/16, 1/16, -1/16, 1/4, 4)/3 )
 
-    expect_equal(iid(e.BT)[,"favorable"], rep(c(-1/8,1/8,1/8,-1/8),3)/3)
-    expect_equal(iid(e.BT)[,"unfavorable"], rep(c(1/8,-1/8,-1/8,1/8),3)/3)
+    expect_equal(getIid(e.BT)[,"favorable"], rep(c(-1/8,1/8,1/8,-1/8),3)/3)
+    expect_equal(getIid(e.BT)[,"unfavorable"], rep(c(1/8,-1/8,-1/8,1/8),3)/3)
 
     ## second order
     BuyseTest.options(order.Hprojection = 2)
@@ -142,11 +141,10 @@ test_that("iid: binary with strata (balanced groups)", {
                        data = d2, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
     
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair,1:12) ## correct ordering of the pairs
     expect_equal(e.BT@covariance, e1.BT@covariance) ## assumes vs. does not assumes binary score when computing second order terms
     expect_equal(e.BT@covariance, e2.BT@covariance)
     
-    expect_equal(as.double(e.BT@covariance), c(5/64,5/64,-3/64, 1/4, 4)/3 )
+    expect_equal(as.double(e.BT@covariance), c(5/64, 5/64, -3/64, 1/4, 4)/3 )
 })
 
 d2.bis <- rbind(cbind(d.bis, strata = 1),
@@ -164,10 +162,10 @@ test_that("iid: binary and no strata (unbalanced groups)", {
                        method.inference = "u-statistic-bebu")
     
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(as.double(e.BT@covariance), c(0,2/27,0,2/27,0)/3 )
+    expect_equal(as.double(e.BT@covariance), c(0, 2/27, 0, 2/27, 0)/3 )
 
-    expect_equal(iid(e.BT)[,"favorable"], rep(c(0,0,0,0),3)/3)
-    expect_equal(iid(e.BT)[,"unfavorable"], rep(c(0,-1/9,-1/9,2/9),3)/3)
+    expect_equal(getIid(e.BT)[,"favorable"], rep(c(0,0,0,0),3)/3)
+    expect_equal(getIid(e.BT)[,"unfavorable"], rep(c(0,-1/9,-1/9,2/9),3)/3)
 
     ## second order
     BuyseTest.options(order.Hprojection = 2)
@@ -181,11 +179,10 @@ test_that("iid: binary and no strata (unbalanced groups)", {
                        data = d2.bis, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
     
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair,1:9) ## correct ordering of the pairs
     expect_equal(e.BT@covariance, e1.BT@covariance) ## assumes vs. does not assumes binary score when computing second order terms
     expect_equal(e.BT@covariance, e2.BT@covariance)
     
-    expect_equal(as.double(e.BT@covariance), c(0,2/27,0,2/27,0)/3 )
+    expect_equal(as.double(e.BT@covariance), c(0, 2/27, 0, 2/27, 0)/3 )
 })
 
 ## ** 1 continuous variable
@@ -223,8 +220,8 @@ test_that("Manual calculation of second order H projection (no strata)",{
     dt.pair[, H2.unfavorable := unfavorable - H1C.unfavorable - H1T.unfavorable]
 
     ## check H1
-    expect_true(all(abs(dt.pair[!duplicated(index.C),.(H1C.favorable/.N,H1C.unfavorable/.N)]-iid(e.BT_c3)[1:n,])<1e-6))
-    expect_true(all(abs(dt.pair[!duplicated(index.T),.(H1T.favorable/.N,H1T.unfavorable/.N)]-iid(e.BT_c3)[(n+1):(2*n),])<1e-6))
+    expect_true(all(abs(dt.pair[!duplicated(index.C),.(H1C.favorable/.N,H1C.unfavorable/.N)]-getIid(e.BT_c3)[1:n,])<1e-6))
+    expect_true(all(abs(dt.pair[!duplicated(index.T),.(H1T.favorable/.N,H1T.unfavorable/.N)]-getIid(e.BT_c3)[(n+1):(2*n),])<1e-6))
         
     ## check H2
     manual <- dt.pair[,.(favorable = var2(H2.favorable)/.N,
@@ -268,8 +265,8 @@ test_that("Manual calculation of second order H projection (strata)",{
     dt.pair[, H2.unfavorable := unfavorable - H1C.unfavorable - H1T.unfavorable]
 
     ## check H1
-    expect_true(all(abs(dt.pair[!duplicated(index.C),.(H1C.favorable/.N,H1C.unfavorable/.N)]-iid(e.BT_c3)[which(dtS$treatment=="C"),])<1e-6))
-    expect_true(all(abs(dt.pair[!duplicated(index.T),.(H1T.favorable/.N,H1T.unfavorable/.N)]-iid(e.BT_c3)[which(dtS$treatment=="T"),])<1e-6))
+    expect_true(all(abs(dt.pair[!duplicated(index.C),.(H1C.favorable/.N,H1C.unfavorable/.N)]-getIid(e.BT_c3)[which(dtS$treatment=="C"),])<1e-6))
+    expect_true(all(abs(dt.pair[!duplicated(index.T),.(H1T.favorable/.N,H1T.unfavorable/.N)]-getIid(e.BT_c3)[which(dtS$treatment=="T"),])<1e-6))
         
     ## check H2
     manual <- dt.pair[,.(favorable = var2(H2.favorable)/.N,
@@ -330,15 +327,13 @@ test_that("iid: two endpoints (no strata - first order)", {
                        data = d, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
 
+    GS <- matrix(c(0.00249999, 0.00304956, 0.00249999, 0.00320223, -0.00249201, -0.00292598, 0.009984, 0.01210375, 0.16025641, 0.07778735), 
+                 nrow = 2, 
+                 ncol = 5, 
+                 dimnames = list(c("toxicity_0.5", "score_1"),c("favorable", "unfavorable", "covariance", "netBenefit", "winRatio")) 
+                 ) 
+    expect_equal(e.BT@covariance, GS)
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(as.double(e.BT@covariance["toxicity_0.5",]),
-                 c(0.002499994, 0.002499994, -0.002492006, 0.009984000, 0.160256410), tol = 1e-6 )
-    expect_equal(as.double(e.BT@covariance["score_1",]),
-                 c(0.003049562, 0.003202234, -0.002925978, 0.012103750, 0.077787351), tol = 1e-6 )
-
-    ## favorable unfavorable   covariance
-    ## toxicity 0.002499994 0.002499994 -0.002492006
-    ## score    0.008041562 0.008194234 -0.002925978
 
     ## same endpoint
     e.BT <- BuyseTest(treatment ~  cont(score, threshold = 2) + cont(score, threshold = 1),
@@ -367,7 +362,7 @@ test_that("iid: two endpoints (no strata - first order)", {
                  c(0.00194112, 0.00143968, -0.00028544, 0.00395168, 0.38625867), tol = 1e-6 )
 
     ## cluster argument
-    expect_equal(unname(iid(e.BT, cluster = 1:NROW(d))), unname(iid(e.BT)), tol = 1e-6)
+    expect_equal(unname(getIid(e.BT, cluster = 1:NROW(d))), unname(getIid(e.BT)), tol = 1e-6)
     expect_equal(confint(e.BT, cluster = 1:NROW(d)),  confint(e.BT), tol = 1e-6)
 })
 
@@ -384,8 +379,8 @@ test_that("iid: two endpoints (no strata - second order)", {
                        data = d, keep.pairScore = TRUE,
                        method.inference = "u-statistic-bebu")
 
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair, 1:2500)
-    expect_equal(e1.BT@tablePairScore[[1]][e1.BT@tablePairScore[[2]]$index.pair,.(index.C,index.T)],
+    remaining.pairs <- e1.BT@tablePairScore[[2]]$index.pair
+    expect_equal(e1.BT@tablePairScore[[1]][index.pair %in% remaining.pairs,.(index.C,index.T)],
                  e1.BT@tablePairScore[[2]][,.(index.C,index.T)])
     expect_equal(e.BT@covariance, e1.BT@covariance)
     expect_equal(e.BT@covariance, e2.BT@covariance)
@@ -407,8 +402,8 @@ test_that("iid: two endpoints (no strata - second order)", {
 
     expect_equal(e.BT@covariance, e1.BT@covariance)
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair, 1:2500)
-    expect_equal(e1.BT@tablePairScore[[1]][e1.BT@tablePairScore[[2]]$index.pair,.(index.C,index.T)],
+    remaining.pairs <- e1.BT@tablePairScore[[2]]$index.pair
+    expect_equal(e1.BT@tablePairScore[[1]][index.pair %in% remaining.pairs,.(index.C,index.T)],
                  e1.BT@tablePairScore[[2]][,.(index.C,index.T)])
     expect_equal(as.double(e.BT@covariance["score_2",]),
                  c(0.000374400, 0.0006309248, -0.000164736, 0.001334797, 0.13361289), tol = 1e-6 )
@@ -428,8 +423,8 @@ test_that("iid: two endpoints (no strata - second order)", {
 
     expect_equal(e.BT@covariance, e1.BT@covariance)
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair, 1:2500)
-    expect_equal(e1.BT@tablePairScore[[1]][e1.BT@tablePairScore[[2]]$index.pair,.(index.C,index.T)],
+    remaining.pairs <- e1.BT@tablePairScore[[2]]$index.pair
+    expect_equal(e1.BT@tablePairScore[[1]][index.pair %in% remaining.pairs,.(index.C,index.T)],
                  e1.BT@tablePairScore[[2]][,.(index.C,index.T)])
     expect_equal(as.double(e.BT@covariance["eventtime_1",]),
                  c(1.126726e-04, 1.183647e-04, -1.522106e-05,  2.614794e-04, 4.680545e-01), tol = 1e-6 )
@@ -466,7 +461,7 @@ test_that("iid: two endpoints (strata)", {
                  c(0.0009537952, 0.001180109, -0.0008541376, 0.0038421792, 0.0444467864), tol = 1e-6 )
 
     ## cluster argument
-    expect_equal(unname(iid(e.BT, cluster = 1:NROW(d2))), unname(iid(e.BT)), tol = 1e-6)
+    expect_equal(unname(getIid(e.BT, cluster = 1:NROW(d2))), unname(getIid(e.BT)), tol = 1e-6)
     expect_equal(confint(e.BT, cluster = 1:NROW(d2)),  confint(e.BT), tol = 1e-6)
 
     ## second order
@@ -483,8 +478,9 @@ test_that("iid: two endpoints (strata)", {
 
     ## expect_equal(e.BT@covariance, e1.BT@covariance, tol = 1e-4) ## imperfect match
     expect_equal(e.BT@covariance, e2.BT@covariance)
-    expect_equal(e1.BT@tablePairScore[[1]]$index.pair, 1:5000)
-    expect_equal(e1.BT@tablePairScore[[1]][e1.BT@tablePairScore[[2]]$index.pair,.(index.C,index.T)],
+    remaining.pairs <- e1.BT@tablePairScore[[2]]$index.pair
+
+    expect_equal(e1.BT@tablePairScore[[1]][index.pair %in% remaining.pairs,.(index.C,index.T)],
                  e1.BT@tablePairScore[[2]][,.(index.C,index.T)])
     expect_equal(as.double(e1.BT@covariance["score2_1",]),
                  c(0.0009675260, 0.0011951397, -0.0008478746, 0.0038584150, 0.0446399926), tol = 1e-6 )
@@ -591,12 +587,12 @@ test_that("iid with nuisance parameters: 1 TTE + 1 binary",{
 
     test <- confint(e.BT_ttebin)
     attr(test,"n.resampling") <- NULL
-    GS <- matrix(c(-0.25, -0.45, 0.1716352, 0.2235767, -0.5471036, -0.77557548, 0.10304552, 0.06467924, 0.1629835, 0.08382162), 
+    GS <- matrix(c(-0.25, -0.45, 0.19367783, 0.22934636, -0.5785771, -0.78116402, 0.14839171, 0.07878577, 0.21633631, 0.0919046), 
                  nrow = 2, 
                  ncol = 5, 
                  dimnames = list(c("eventtime_1", "toxicity_0.5"),c("estimate", "se", "lower.ci", "upper.ci", "p.value")) 
                  ) 
-    expect_equal(test,GS, tol = 1e-6)
+    expect_equal(test, GS, tol = 1e-6)
 
     ## GS <- BuyseTest(treatment ~ tte(eventtime, status, threshold = 1) + bin(toxicity),
                     ## data = dt, 
@@ -615,41 +611,38 @@ test_that("iid with nuisance parameters: 2 TTE",{
                            argsCont = list(mu.T = 1:3, sigma.T = rep(1,3)),
                            argsTTE = list(rates.T = 1/(1:3), rates.Censoring.T = rep(1,3)))
     setkeyv(dt.sim,c("treatment","eventtime1"))
-    dt.sim[,status1.bis := c(status1[1:(.N-1)],1),by="treatment"] ## make sure last observation is a case
+    ## dt.sim[,status1.bis := c(status1[1:(.N-1)],1),by="treatment"] ## make sure last observation is a case
 
     ## plot(prodlim(Hist(eventtime1,status1.bis) ~ treatment, data = dt.sim))
     e.BT_tte1 <- BuyseTest(treatment ~ tte(eventtime2, status2, threshold = 1),
                            data = dt.sim, 
                            method.inference = "u-statistic")
-    e.BT_tte2 <- BuyseTest(treatment ~ tte(eventtime1, status1.bis, threshold = 1e5) + tte(eventtime2, status2, threshold = 1),
-                           data = dt.sim, 
-                           method.inference = "u-statistic")
+    ## e.BT_tte2 <- BuyseTest(treatment ~ tte(eventtime1, status1.bis, threshold = 1e5) + tte(eventtime2, status2, threshold = 1),
+                           ## data = dt.sim, 
+                           ## method.inference = "u-statistic")
     e.BT_tte3 <- BuyseTest(treatment ~ tte(eventtime1, status1, threshold = 1e5) + tte(eventtime2, status2, threshold = 1),
                            data = dt.sim, 
                            method.inference = "u-statistic")
 
-    expect_equal(e.BT_tte1@count.favorable[1], e.BT_tte2@count.favorable[2])
-    expect_equal(e.BT_tte1@count.unfavorable[1], e.BT_tte2@count.unfavorable[2])
-    expect_equal(e.BT_tte1@count.neutral[1], e.BT_tte2@count.neutral[2])
-    expect_equal(e.BT_tte1@count.uninf[1], e.BT_tte2@count.uninf[2])
-
-    expect_equal(e.BT_tte1@covariance[1,],e.BT_tte2@covariance[2,], tol = 1e-6)
-    ## only approx because of the uncertainty related to the incomplete knoweldge of the survival curves
-    expect_equal(e.BT_tte1@covariance[1,],e.BT_tte3@covariance[2,], tol = 1e-3)
+    expect_equal(e.BT_tte1@count.favorable[1], e.BT_tte3@count.favorable[2])
+    expect_equal(e.BT_tte1@count.unfavorable[1], e.BT_tte3@count.unfavorable[2])
+    expect_equal(e.BT_tte1@count.neutral[1], e.BT_tte3@count.neutral[2])
+    expect_equal(e.BT_tte1@count.uninf[1], e.BT_tte3@count.uninf[2])
 
+    ## expect_equal(e.BT_tte1@covariance[1,],e.BT_tte3@covariance[2,], tol = 1e-6)
 
     e.BT_tte <- BuyseTest(treatment ~ tte(eventtime1, status1, threshold = 1) + tte(eventtime2, status2, threshold = 1) + bin(toxicity1),
                           data = dt.sim, 
                           method.inference = "u-statistic")
     test <- confint(e.BT_tte)
     attr(test,"n.resampling") <- NULL
-    GS <- matrix(c(-0.25168271, -0.11821275, -0.15513379, 0.14101332, 0.21861747, 0.21603064, -0.50222756, -0.50300659, -0.53007553, 0.03784555, 0.30569245, 0.27054988, 0.08755073, 0.59218176, 0.47985706), 
+    GS <- matrix(c(-0.25168271, -0.11821275, -0.15513379, 0.1363316, 0.21272703, 0.21392362, -0.49486602, -0.49420898, -0.52702616, 0.0280597, 0.29504029, 0.26662361, 0.07720623, 0.5819696, 0.47554172), 
                  nrow = 3, 
                  ncol = 5, 
                  dimnames = list(c("eventtime1_1", "eventtime2_1", "toxicity1_0.5"),c("estimate", "se", "lower.ci", "upper.ci", "p.value")) 
-                 )
-    expect_equal(test,GS, tol = 1e-6)
+                 ) 
 
+    expect_equal(test, GS, tol = 1e-3)
 })
 
 ## * normalization iid
@@ -662,12 +655,13 @@ test_that("iid - remove normalization", {
     e.all <- BuyseTest(treatment~bin(toxicity)+cont(score)+strata,
                        method.inference = "u-statistic",
                        data = dt, trace = 0)
+    
     e.strata <- BuyseTest(treatment~bin(toxicity)+cont(score)+strata,
                           method.inference = "u-statistic",
                           data = dt[strata=="a"], trace = 0)
 
-    iid.all <- iid(e.all, normalize = FALSE)[which(dt$strata=="a"),]
-    iid.strata <- iid(e.strata, normalize = FALSE)
+    iid.all <- getIid(e.all, normalize = FALSE)[which(dt$strata=="a"),]
+    iid.strata <- getIid(e.strata, normalize = FALSE)
 
     expect_equal(unname(iid.all),unname(iid.strata), tol = 1e-9)
 })
